#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ogl=new OGLWdgt(ui->widget);
    oglpp=new OGLPP(ui->wgPP);
    oglnb=new oglnbody(ui->wgNBody);

    ogl->resize(ui->widget->width(),ui->widget->height());
    oglpp->resize(ui->wgPP->width(),ui->wgPP->height());
    oglnb->resize(ui->wgNBody->width(),ui->wgNBody->height());

    oclinfo=new oclInfo(this);
    oglinf=new oglinfo(this);

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), ogl, SLOT(animate()));
    timer->start(1);

    timerfr = new QTimer(this);
    connect(timerfr, SIGNAL(timeout()), ogl, SLOT(framecount()));
    timerfr->start(1000);

    timerpp = new QTimer(this);
    connect(timerpp, SIGNAL(timeout()), oglpp, SLOT(animate()));
    timerpp->start(1);

    timerppfr = new QTimer(this);
    connect(timerppfr, SIGNAL(timeout()), oglpp, SLOT(framecount()));
    timerppfr->start(1000);

    timernb = new QTimer(this);
    connect(timernb, SIGNAL(timeout()), oglnb, SLOT(animate()));
    timernb->start(1);

    timernbfr = new QTimer(this);
    connect(timernbfr, SIGNAL(timeout()), oglnb, SLOT(framecount()));
    timernbfr->start(1000);

    ui->horizontalSlider->setValue(10);
    ui->hsFreq->setValue(40);
    this->setWindowTitle("OpenCL Simples");

    //ui->tab_3->setVisible(false);

    ZeroMemory(&noclInf,sizeof(noclInf));
    if (_noclGetInfo(&noclInf)==CL_SUCCESS)
    {
        for (unsigned int i=0;i<noclInf.uiNumP;i++)
        {
            ui->cmbPlatform->addItem(noclInf.noclPlt[i].Name);
            //for (unsigned int j=0;j<noclInf.noclPlt[0].uiNumD;j++)
            //{
            //    ui->cmbDevice->addItem(noclInf.noclPlt[i].noclDev[j].Name);
            //}
        }
    }
}



MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
void MainWindow::resizeEvent(QResizeEvent * e)
{
    switch (e->type())
    {
        case QEvent::Resize:
            ogl->resize(ui->widget->width(),ui->widget->height());
            oglpp->resize(ui->wgPP->width(),ui->wgPP->height());
            oglnb->resize(ui->wgNBody->width(),ui->wgNBody->height());
            break;
        default:
            break;
    }
}

void MainWindow::on_pushButton_2_clicked()
{
    if (ui->cmbPlatform->currentIndex()>=0)
    {
        this->uiCurPlat=ui->cmbPlatform->currentIndex();
        ogl->uiCurPlat=ui->cmbPlatform->currentIndex();
    }
    else
    {
        this->uiCurDev=-1;
        ogl->uiCurDev=-1;
    }
    if (ui->cmbDevice->currentIndex()>=0)
    {
        this->uiCurDev=ui->cmbDevice->currentIndex();
        ogl->uiCurDev=ui->cmbDevice->currentIndex();
    }
    else
    {
        this->uiCurDev=-1;
        ogl->uiCurDev=-1;
    }
    ogl->GenInit();
    ogl->qlsd=ui->lcdFC;
    oglpp->qlsd=NULL;
    oglnb->qlsd=NULL;
}

void MainWindow::on_horizontalSlider_valueChanged(int value)
{
    ogl->fanimspeed=(float)value/200+0.001;
}

void MainWindow::on_tabWidget_selected(QString str)
{
    if (str=="OpenGL post-processing")
    {
        timerpp->start(1);
        timer->stop();
        timernb->stop();
        if ((oglpp->uiCurDev!=-1) && (oglpp->uiCurPlat!=-1))
        {
            ui->cmbPlatform->setCurrentIndex(oglpp->uiCurPlat);
            ui->cmbDevice->setCurrentIndex(oglpp->uiCurDev);
        }
        oglpp->qlsd=ui->lcdFC;
        ogl->qlsd=NULL;
        oglnb->qlsd=NULL;
    }
    if (str=="OpenCL Simple OpenGL Interop")
    {
        timer->start(1);
        timernb->stop();
        timerpp->stop();
        if ((ogl->uiCurDev!=-1) && (ogl->uiCurPlat!=-1))
        {
            ui->cmbPlatform->setCurrentIndex(ogl->uiCurPlat);
            ui->cmbDevice->setCurrentIndex(ogl->uiCurDev);
        }
        ogl->qlsd=ui->lcdFC;
        oglpp->qlsd=NULL;
        oglnb->qlsd=NULL;
    }
    if (str=="OpenCL N-Body Physics Simulation")
    {
        timernb->start(1);
        timer->stop();
        timerpp->stop();
        if ((oglnb->uiCurDev!=-1) && (oglnb->uiCurPlat!=-1))
        {
            ui->cmbPlatform->setCurrentIndex(oglnb->uiCurPlat);
            ui->cmbDevice->setCurrentIndex(oglnb->uiCurDev);
        }
        ogl->qlsd=NULL;
        oglpp->qlsd=NULL;
        oglnb->qlsd=ui->lcdFC;
    }
    ogl->resize(ui->widget->width(),ui->widget->height());
    oglpp->resize(ui->wgPP->width(),ui->wgPP->height());
    oglnb->resize(ui->wgNBody->width(),ui->wgNBody->height());
}

void MainWindow::on_hsFreq_valueChanged(int value)
{
    ogl->ffreq=(float)value/10;
}

void MainWindow::OpenFrmInfo()
{
    oclinfo->show();
}
void MainWindow::OpenFrmOGLInfo()
{
    oglinf->show();
}
void MainWindow::on_pbPPStart_clicked()
{
    if (ui->cmbPlatform->currentIndex()>=0)
    {
        this->uiCurPlat=ui->cmbPlatform->currentIndex();
        oglpp->uiCurPlat=ui->cmbPlatform->currentIndex();
    }
    else
    {
        this->uiCurDev=-1;
        oglpp->uiCurDev=-1;
    }
    if (ui->cmbDevice->currentIndex()>=0)
    {
        this->uiCurDev=ui->cmbDevice->currentIndex();
        oglpp->uiCurDev=ui->cmbDevice->currentIndex();
    }
    else
    {
        this->uiCurDev=-1;
        oglpp->uiCurDev=-1;
    }
    oglpp->GenInit();
    oglpp->qlsd=ui->lcdFC;
    ogl->qlsd=NULL;
    oglnb->qlsd=NULL;
}

void MainWindow::on_cmbPlatform_currentIndexChanged(int index)
{
    ui->cmbDevice->clear();
    for (unsigned int j=0;j<noclInf.noclPlt[index].uiNumD;j++)
    {
        ui->cmbDevice->addItem(noclInf.noclPlt[index].noclDev[j].Name);
    }
}

void MainWindow::on_cmbDevice_currentIndexChanged(int index)
{

}

void MainWindow::on_pbNBody_clicked()
{
    if (ui->cmbPlatform->currentIndex()>=0)
    {
        this->uiCurPlat=ui->cmbPlatform->currentIndex();
        oglnb->uiCurPlat=ui->cmbPlatform->currentIndex();
    }
    else
    {
        this->uiCurDev=-1;
        oglnb->uiCurPlat=-1;
    }
    if (ui->cmbDevice->currentIndex()>=0)
    {
        this->uiCurDev=ui->cmbDevice->currentIndex();
        oglnb->uiCurDev=ui->cmbDevice->currentIndex();
    }
    else
    {
        this->uiCurDev=-1;
        oglnb->uiCurDev=-1;
    }
    oglnb->GenInit();
    ogl->qlsd=NULL;
    oglpp->qlsd=NULL;
    oglnb->qlsd=ui->lcdFC;
}
