#include "oclinfo.h"
#include "ui_oclinfo.h"

/* cl_channel_type */
#define CL_SNORM_INT8                               0x10D0
#define CL_SNORM_INT16                              0x10D1
#define CL_UNORM_INT8                               0x10D2
#define CL_UNORM_INT16                              0x10D3
#define CL_UNORM_SHORT_565                          0x10D4
#define CL_UNORM_SHORT_555                          0x10D5
#define CL_UNORM_INT_101010                         0x10D6
#define CL_SIGNED_INT8                              0x10D7
#define CL_SIGNED_INT16                             0x10D8
#define CL_SIGNED_INT32                             0x10D9
#define CL_UNSIGNED_INT8                            0x10DA
#define CL_UNSIGNED_INT16                           0x10DB
#define CL_UNSIGNED_INT32                           0x10DC
#define CL_HALF_FLOAT                               0x10DD
#define CL_FLOAT                                    0x10DE

#define getChannelType(channelt,strChannelType) \
        if (channelt==0x10D0) strChannelType="CL_SNORM_INT8";\
        if (channelt==0x10D1) strChannelType="CL_SNORM_INT16";\
        if (channelt==0x10D2) strChannelType="CL_UNORM_INT8";\
        if (channelt==0x10D3) strChannelType="CL_UNORM_INT16";\
        if (channelt==0x10D4) strChannelType="CL_UNORM_SHORT_565";\
        if (channelt==0x10D5) strChannelType="CL_UNORM_SHORT_555";\
        if (channelt==0x10D6) strChannelType="CL_UNORM_INT_101010";\
        if (channelt==0x10D7) strChannelType="CL_SIGNED_INT8";\
        if (channelt==0x10D8) strChannelType="CL_SIGNED_INT16";\
        if (channelt==0x10D9) strChannelType="CL_SIGNED_INT32";\
        if (channelt==0x10DA) strChannelType="CL_UNSIGNED_INT8";\
        if (channelt==0x10DB) strChannelType="CL_UNSIGNED_INT16";\
        if (channelt==0x10DC) strChannelType="CL_UNSIGNED_INT32";\
        if (channelt==0x10DD) strChannelType="CL_HALF_FLOAT";\
        if (channelt==0x10DE) strChannelType="CL_FLOAT";

#define getChannelOrder(channel,strChannel) \
        if (channel==0x10B0) strChannel="CL_R";\
        if (channel==0x10B1) strChannel="CL_A";\
        if (channel==0x10B2) strChannel="CL_RG";\
        if (channel==0x10B3) strChannel="CL_RA";\
        if (channel==0x10B4) strChannel="CL_RGB";\
        if (channel==0x10B5) strChannel="CL_RGBA";\
        if (channel==0x10B6) strChannel="CL_BGRA";\
        if (channel==0x10B7) strChannel="CL_ARGB";\
        if (channel==0x10B8) strChannel="CL_INTENSITY";\
        if (channel==0x10B9) strChannel="CL_LUMINANCE";

oclInfo::oclInfo(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::oclInfo)
{
    ui->setupUi(this);
    resize(768,512);

    ui->twInfo->setColumnWidth(0,340);

    memset(&noclInf,0,sizeof(noclInf));
    if (_noclGetInfo(&noclInf)==CL_SUCCESS)
    {
        for (unsigned int i=0;i<noclInf.uiNumP;i++)
        {
            QTreeWidgetItem *ptwg=new QTreeWidgetItem(ui->twInfo);
            QTreeWidgetItem *ptwgSub=0;
            QTreeWidgetItem *ptwgDev=0;
            ptwg->setText(0,noclInf.noclPlt[i].Name);
            ptwg->setBackgroundColor(0,QColor(0xdb,0xde,0xdf));
            ptwg->setExpanded(true);
            ptwgSub=new QTreeWidgetItem(ptwg);
            ptwgSub->setText(0,"Version: "+QString(noclInf.noclPlt[i].Ver));
            ptwgSub=new QTreeWidgetItem(ptwg);
            ptwgSub->setText(0,"Vendor: "+QString(noclInf.noclPlt[i].Vendor));
            ptwgSub=new QTreeWidgetItem(ptwg);
            ptwgSub->setText(0,"Profile: "+QString(noclInf.noclPlt[i].Profile));
            ptwgSub=new QTreeWidgetItem(ptwg);
            ptwgSub->setText(0,"Extensions:\t"+QString(noclInf.noclPlt[i].Ext).trimmed().replace(" ","\n\t"));
            for (unsigned int j=0;j<noclInf.noclPlt[i].uiNumD;j++)
            {
                ptwgSub=new QTreeWidgetItem(ptwg);
                ptwgSub->setText(0,"Device: "+QString(noclInf.noclPlt[i].noclDev[j].Name));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"Vendor: "+QString(noclInf.noclPlt[i].noclDev[j].Vendor));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"Version: "+QString(noclInf.noclPlt[i].noclDev[j].Ver));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                if (noclInf.noclPlt[i].noclDev[j].type==1) ptwgDev->setText(1,"Type: "+QString("CL_DEVICE_TYPE_DEFAULT"));
                if (noclInf.noclPlt[i].noclDev[j].type==2) ptwgDev->setText(1,"Type: "+QString("CL_DEVICE_TYPE_CPU"));
                if (noclInf.noclPlt[i].noclDev[j].type==4) ptwgDev->setText(1,"Type: "+QString("CL_DEVICE_TYPE_GPU"));
                if (noclInf.noclPlt[i].noclDev[j].type==8) ptwgDev->setText(1,"Type: "+QString("CL_DEVICE_TYPE_ACCELERATOR"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_COMPUTE_UNITS: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_compute_units));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_work_item_dimensions));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_WORK_ITEM_SIZES: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_work_item_sizes[0])+"x"+QString().number(noclInf.noclPlt[i].noclDev[j].max_work_item_sizes[1])+"x"+QString().number(noclInf.noclPlt[i].noclDev[j].max_work_item_sizes[2]));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_WORK_GROUP_SIZE: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_work_group_size));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_CLOCK_FREQUENCY: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_clock_frequency)+" MHz");
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_ADDRESS_BITS: "+QString().number(noclInf.noclPlt[i].noclDev[j].address_bits));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_MEM_ALLOC_SIZE: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_mem_alloc_size/1024/1024)+" MByte");
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_GLOBAL_MEM_SIZE: "+QString().number(noclInf.noclPlt[i].noclDev[j].global_mem_size/1024/1024)+" MByte");
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                if (noclInf.noclPlt[i].noclDev[j].error_correction_support==0) ptwgDev->setText(1,"CL_DEVICE_ERROR_CORRECTION_SUPPORT: "+QString("no"));
                if (noclInf.noclPlt[i].noclDev[j].error_correction_support>0) ptwgDev->setText(1,"CL_DEVICE_ERROR_CORRECTION_SUPPORT: "+QString("yes"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                if (noclInf.noclPlt[i].noclDev[j].local_mem_type==1) ptwgDev->setText(1,"CL_DEVICE_LOCAL_MEM_TYPE: "+QString("local"));
                if (noclInf.noclPlt[i].noclDev[j].local_mem_type==2) ptwgDev->setText(1,"CL_DEVICE_LOCAL_MEM_TYPE: "+QString("global"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_LOCAL_MEM_SIZE: "+QString().number(noclInf.noclPlt[i].noclDev[j].local_mem_size/1024)+" KByte");
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_constant_buffer_size/1024)+" KByte");
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                QString strQueue;
                strQueue.clear();
                if ((noclInf.noclPlt[i].noclDev[j].queue_properties & 1)==1) strQueue+=" CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE";
                if ((noclInf.noclPlt[i].noclDev[j].queue_properties & 2)==2) strQueue+=" CL_QUEUE_PROFILING_ENABLE ";
                ptwgDev->setText(1,"CL_DEVICE_QUEUE_PROPERTIES: "+strQueue.trimmed().replace(" ","\n\t"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                if (noclInf.noclPlt[i].noclDev[j].image_support>0) ptwgDev->setText(1,"CL_DEVICE_IMAGE_SUPPORT: "+QString("yes"));
                if (noclInf.noclPlt[i].noclDev[j].image_support==0) ptwgDev->setText(1,"CL_DEVICE_IMAGE_SUPPORT: "+QString("no"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_READ_IMAGE_ARGS: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_read_image_args));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_MAX_WRITE_IMAGE_ARGS: "+QString().number(noclInf.noclPlt[i].noclDev[j].max_write_image_args));
                QString strFpConf;
                strFpConf.clear();
                if ((noclInf.noclPlt[i].noclDev[j].single_fp_config & 1)==1) strFpConf+=" CL_FP_DENORM";
                if ((noclInf.noclPlt[i].noclDev[j].single_fp_config & 2)==2) strFpConf+=" CL_FP_INF_NAN";
                if ((noclInf.noclPlt[i].noclDev[j].single_fp_config & 4)==4) strFpConf+=" CL_FP_ROUND_TO_NEAREST";
                if ((noclInf.noclPlt[i].noclDev[j].single_fp_config & 8)==8) strFpConf+=" CL_FP_ROUND_TO_ZERO";
                if ((noclInf.noclPlt[i].noclDev[j].single_fp_config & 16)==16) strFpConf+=" CL_FP_ROUND_TO_INF";
                if ((noclInf.noclPlt[i].noclDev[j].single_fp_config & 32)==32) strFpConf+=" CL_FP_FMA";
                ptwgDev->setText(1,"CL_DEVICE_SINGLE_FP_CONFIG: "+strFpConf.trimmed().replace(" ","\n\t"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_SINGLE_FP_CONFIG: "+QString().number(noclInf.noclPlt[i].noclDev[j].single_fp_config));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_IMAGE2D_MAX_WIDTH: "+QString().number(noclInf.noclPlt[i].noclDev[j].image2d_max_width));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_IMAGE2D_MAX_HEIGHT: "+QString().number(noclInf.noclPlt[i].noclDev[j].image2d_max_height));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_IMAGE3D_MAX_WIDTH: "+QString().number(noclInf.noclPlt[i].noclDev[j].image3d_max_width));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_IMAGE3D_MAX_HEIGHT: "+QString().number(noclInf.noclPlt[i].noclDev[j].image3d_max_height));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"CL_DEVICE_IMAGE3D_MAX_DEPTH: "+QString().number(noclInf.noclPlt[i].noclDev[j].image3d_max_depth));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"Extensions:\t"+QString(noclInf.noclPlt[i].noclDev[j].Ext).trimmed().replace(QRegExp(" {1,2}"),"\n\t"));
                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"2D Image Formats Supported:\t"+QString().number(noclInf.noclPlt[i].noclDev[j].uiSup2D));

                QString strChan;
                QString strChanType;
                for (int k=0;k<noclInf.noclPlt[i].noclDev[j].uiSup2D;k++)
                {
                    ptwgDev=new QTreeWidgetItem(ptwgSub);
                    getChannelOrder(noclInf.noclPlt[i].noclDev[j].imgFormats[k].image_channel_order,strChan);
                    getChannelType(noclInf.noclPlt[i].noclDev[j].imgFormats[k].image_channel_data_type,strChanType);
                    ptwgDev->setText(1,QString("%1%2").arg(strChan,-20).arg(strChanType));
                }

                ptwgDev=new QTreeWidgetItem(ptwgSub);
                ptwgDev->setText(1,"3D Image Formats Supported:\t"+QString().number(noclInf.noclPlt[i].noclDev[j].uiSup3D));

                for (int k=0;k<noclInf.noclPlt[i].noclDev[j].uiSup3D;k++)
                {
                    ptwgDev=new QTreeWidgetItem(ptwgSub);
                    getChannelOrder(noclInf.noclPlt[i].noclDev[j].imgFormats3D[k].image_channel_order,strChan);
                    getChannelType(noclInf.noclPlt[i].noclDev[j].imgFormats3D[k].image_channel_data_type,strChanType);
                    ptwgDev->setText(1,QString("%1%2").arg(strChan,-20).arg(strChanType));
                }

            }
        }
    }
}

oclInfo::~oclInfo()
{
    delete ui;
}

void oclInfo::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void oclInfo::on_pbClose_clicked()
{
    this->close();
}


