#include "oglnbody.h"

#define ERRORVOID(err) Cleanup(err); if(err!=CL_SUCCESS) return;

oglnbody::oglnbody(QWidget *parent) :
    QGLWidget(parent)
{
    fxRot=0.0f;
    fyRot=0.0f;
    fzRot=0.0f;
    fx=0.0f;
    fzTr=-0.5f;
    cliNumPart=1024;
    initPos=NULL;
    initVel=NULL;
    Pos=NULL;
    Vel=NULL;
    delT=0.005;
    espSqr=50;
    iErr=0;
    clk=NULL;
    clp=NULL;
    clcq=NULL;
    clc=NULL;
    updatedPos=NULL;

    uiCurDev=-1;
    uiCurPlat=-1;

    iCountSec=0;

    memset(&noclInf,0,sizeof(noclInf));
    if (_noclGetInfo(&noclInf)==CL_SUCCESS)
    {
    }
    qlsd=NULL;
}


void oglnbody::initializeGL()
{
    qglClearColor(Qt::black);
    glClearColor(0.0 ,0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT);
    glClear(GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
}

void oglnbody::resizeGL(int nWidth,int nHeight)
{
    glViewport(0,0,(GLsizei)nWidth,(GLsizei)nHeight);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, this->width()/this->height(),0.1, 1000.0);
    gluLookAt (0.0, 0.0, -2.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0);
}

void oglnbody::draw(float fxx)
{
    fx=fxx;
    this->updateGL();
}

void oglnbody::paintGL()
{
    if (fx==0)
    {
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    }
    if (fx==1)
    {
        glClearColor(0.0 ,0.0, 0.0, 0.0);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glPointSize(1.0);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE);
        glEnable(GL_BLEND);
        glDepthMask(GL_FALSE);
        glColor3f(1.0f,0.6f,0.0f);

        this->RunKernel();

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glTranslatef(0,0,fzTr);
        glRotatef(fxRot,1.0,0.0,0.0);
        glRotatef(fyRot,0.0,1.0,0.0);
        glRotatef(fzRot,0.0,0.0,1.0);
        glBegin(GL_POINTS);
            for(int i=0; i < this->cliNumPart; ++i)
            {
                glVertex3d(Pos[i*4+ 0]/300, Pos[i*4+1]/300, Pos[i*4+2]/300);
            }
        glEnd();

        glFlush();

    }
}

void oglnbody::wheelEvent(QWheelEvent * qwe)
{
    if (qwe->delta()>0) fzTr+=0.1;
    if (qwe->delta()<0) fzTr-=0.1;
    this->repaint();;
}

void oglnbody::mousePressEvent(QMouseEvent *qme)
{
    qpoint=qme->pos();
    this->repaint();;
}

void oglnbody::mouseMoveEvent(QMouseEvent *qme)
{
    fxRot+=180*(GLfloat)(qme->y()-qpoint.y())/this->height();
    fyRot+=180*(GLfloat)(qme->x()-qpoint.x())/this->width();
    this->repaint();;

    qpoint=qme->pos();
}

void oglnbody::animate()
{
    this->repaint();
}

void oglnbody::GenInit()
{
    if ((this->uiCurPlat<0) || (this->uiCurDev<0)) return;
    this->Cleanup(-500);

    initPos = (cl_float*)malloc(this->cliNumPart * sizeof(cl_float4));
    if(initPos == NULL) return;
    initVel = (cl_float*)malloc(this->cliNumPart * sizeof(cl_float4));
    if(initVel == NULL) return;
    Pos = (cl_float*)malloc(this->cliNumPart * sizeof(cl_float4));
    if(Pos == NULL) return;
    Vel = (cl_float*)malloc(this->cliNumPart * sizeof(cl_float4));
    if(Vel == NULL) return;

    for(int i = 0; i < this->cliNumPart; ++i)
    {
        int index = 4 * i;
        for(int j = 0; j < 3; ++j)
        {
            initPos[index + j] = ((1.0f - (float)rand()/(float)RAND_MAX) * 3 + (float)rand()/(float)RAND_MAX *50);
        }

        initPos[index + 3] = ((1.0f - (float)rand()/(float)RAND_MAX) * 1 + (float)rand()/(float)RAND_MAX *1000);

        for(int j = 0; j < 3; ++j)
        {
            initVel[index + j] = 0.0f;
        }

        initVel[3] = 0.0f;
    }

    memcpy(Pos, initPos, 4 * this->cliNumPart * sizeof(cl_float));
    memcpy(Vel, initVel, 4 * this->cliNumPart * sizeof(cl_float));

    cl_context_properties props[]=
    {
        CL_CONTEXT_PLATFORM,(cl_context_properties)noclInf.clpi[this->uiCurPlat],
        0
    };
    clc=clCreateContext(props,1,&noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev],NULL,NULL,&iErr);
    ERRORVOID(iErr);

    clcq=clCreateCommandQueue(clc,noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev],0,&iErr);
    ERRORVOID(iErr);

    QFile qfile(":/cl/NBody_Kernels.cl");
    if (!qfile.open(QIODevice::ReadOnly))
    {
        return;
    }

    char *cSource;
    QByteArray qba=qfile.readAll();
    cSource=qba.data();
    size_t lenSource=qba.length();
    qfile.close();

    clp=clCreateProgramWithSource(clc,1,(const char **) &cSource,&lenSource,&iErr);
    ERRORVOID(iErr);

    iErr=clBuildProgram(clp, 0, NULL, "-cl-fast-relaxed-math", NULL, NULL);
    ERRORVOID(iErr);

    clk=clCreateKernel(clp,"nbody_sim",&iErr);
    ERRORVOID(iErr);

    updatedPos=clCreateBuffer(clc,CL_MEM_READ_WRITE | CL_MEM_USE_HOST_PTR,this->cliNumPart*sizeof(cl_float4),Pos,&iErr);
    updatedVel=clCreateBuffer(clc,CL_MEM_READ_WRITE | CL_MEM_USE_HOST_PTR,this->cliNumPart*sizeof(cl_float4),Vel,&iErr);

    iErr=clSetKernelArg(clk,0,sizeof(cl_mem),(void *)&updatedPos);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,1,sizeof(cl_mem),(void *)&updatedVel);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,2,sizeof(cl_int),(void *)&this->cliNumPart);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,3,sizeof(cl_float),(void *)&this->delT);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,4,sizeof(cl_float),(void *)&this->espSqr);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,5,256 * 4 * sizeof(float),NULL);
    ERRORVOID(iErr);

    this->draw(1.0f);

}

void oglnbody::RunKernel()
{
    iCountSec++;

    cl_event events[1];
    size_t globalThreads[] = {this->cliNumPart};
    size_t localThreads[] = {256};

    iErr=clEnqueueNDRangeKernel(clcq,clk,1,NULL,globalThreads, localThreads, 0,0,0);
    ERRORVOID(iErr);

    iErr=clFinish(clcq);
    ERRORVOID(iErr);

    iErr=clEnqueueReadBuffer(clcq,updatedPos,CL_TRUE,0,this->cliNumPart*sizeof(cl_float4), Pos, 0, NULL, &events[0]);
    ERRORVOID(iErr);

    iErr=clWaitForEvents(1, &events[0]);
    ERRORVOID(iErr);

    iErr=clReleaseEvent(events[0]);
    ERRORVOID(iErr);

}

cl_int oglnbody::Cleanup(cl_int iErr)
{

    if (iErr<0)
    {
        if (iErr!=-500)
        {
            _noclGetNameErr(iErr,cErr);
            QMessageBox::critical(this,"Error!",QString(cErr));
        }
        if(clk)clReleaseKernel(clk);
        if(clp)clReleaseProgram(clp);
        if(updatedPos) clReleaseMemObject(updatedPos);
        if(clcq)clReleaseCommandQueue(clcq);
        if(clc)clReleaseContext(clc);
        if(initPos)
        {
            free(initPos);
            initPos = NULL;
        }
        if(initVel)
        {
            free(initVel);
            initVel = NULL;
        }
        if(Pos)
        {
            free(Pos);
            Pos = NULL;
        }
        if(Vel)
        {
            free(Vel);
            Vel = NULL;
        }
    }
    return iErr;
}

void oglnbody::framecount()
{
    if (qlsd) qlsd->display(iCountSec);
    iCountSec=0;
}
