#include "oglpp.h"

#define ERRORVOID(err) Cleanup(err); if(err!=CL_SUCCESS) return;

OGLPP::OGLPP(QWidget *parent) :
    QGLWidget(parent)
{
    fxRot=0.0f;
    fyRot=0.0f;
    fzRot=0.0f;
    fx=0.0f;
    fanim=0.0f;
    fzTr=-2.0f;
    brun=false;
    iErr=0;
    cl_pbo[0]=0;
    cl_pbo[1]=0;
    clk=NULL;
    clp=NULL;
    clcq=NULL;
    clc=NULL;

    uiCurDev=-1;
    uiCurPlat=-1;

    iCountSec=0;

    memset(&noclInf,0,sizeof(noclInf));
    if (_noclGetInfo(&noclInf)==CL_SUCCESS)
    {
    }

    qlsd=NULL;
}

void OGLPP::initializeGL()
{
    qglClearColor(Qt::black);
    LoadFun(&glewFun,this);

    glClearColor(0, 0, 0, 1.0);
    glDisable(GL_DEPTH_TEST);

    glViewport(0, 0, this->width(),this->height());

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, (GLfloat)this->width()/ (GLfloat) this->height(), 0.1, 10.0);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glEnable(GL_LIGHT0);
    float red[] = { 1.0, 0.1, 0.1, 1.0 };
    float white[] = { 1.0, 1.0, 1.0, 1.0 };
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, red);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 60.0);

    quad=gluNewQuadric();
    gluQuadricDrawStyle(quad,GLU_FILL);
    gluQuadricNormals(quad,GLU_SMOOTH);
    gluQuadricTexture(quad,GL_TRUE);
}

void OGLPP::resizeGL(int nWidth,int nHeight)
{
    glBindTexture(GL_TEXTURE_2D, texscreen);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,nWidth, nHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

    image_width=nWidth;
    image_height=nHeight;

    unsigned int size = image_width * image_height * 4 * sizeof(float);

    if(cl_pbo[0]!=0)
    {
        glewFun.BindBuffer(GL_ARRAY_BUFFER,pbos);
        glewFun.BufferData(GL_ARRAY_BUFFER,size,NULL, GL_DYNAMIC_DRAW);
        glewFun.BindBuffer(GL_ARRAY_BUFFER,0);

        glewFun.BindBuffer(GL_ARRAY_BUFFER,pbod);
        glewFun.BufferData(GL_ARRAY_BUFFER,size,NULL, GL_DYNAMIC_DRAW);
        glewFun.BindBuffer(GL_ARRAY_BUFFER,0);

        clReleaseMemObject(cl_pbo[0]);
        clReleaseMemObject(cl_pbo[1]);

        cl_pbo[0]=clCreateFromGLBuffer(clc,CL_MEM_READ_ONLY,pbos,&iErr);
        ERRORVOID(iErr);
        cl_pbo[1]=clCreateFromGLBuffer(clc,CL_MEM_WRITE_ONLY,pbod,&iErr);
        ERRORVOID(iErr);

        iErr=clSetKernelArg(clk,0,sizeof(cl_mem),(void *)&(cl_pbo[0]));
        ERRORVOID(iErr);
        iErr=clSetKernelArg(clk,1,sizeof(cl_mem),(void *)&(cl_pbo[1]));
        ERRORVOID(iErr);
        iErr=clSetKernelArg(clk,2,sizeof(unsigned int),(void *)&image_width);
        ERRORVOID(iErr);
        iErr=clSetKernelArg(clk,3,sizeof(unsigned int),(void *)&image_height);
        ERRORVOID(iErr);

    }
}

void OGLPP::draw(float fxx)
{
    fx=fxx;
    this->updateGL();
}

void OGLPP::paintGL()
{
    if (fx==0)
    {
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
        
    }
    if (fx==1)
    {
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        gluPerspective(60.0, image_width/image_height,0.1, 10.0);

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glTranslatef(0,0,fzTr);
        glRotatef(fxRot,1.0,0.0,0.0);
        glRotatef(fyRot,0.0,1.0,0.0);
        glRotatef(fzRot,0.0,0.0,1.0);

        glViewport(0,0,(GLsizei)image_width,(GLsizei)image_height);

        glEnable(GL_LIGHTING);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LESS);

        float blue[] = { 0.1, 0.1, 1.0, 1.0 };
        float white[] = { 1.0, 1.0, 1.0, 1.0 };
        float green[] = { 0.1, 1.0, 0.1, 1.0 };
        float red[] = { 1.0, 0.1, 0.1, 1.0 };

        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, red);
        glTranslatef(0.5,0,0);
        gluSphere(quad,0.3,100,100);
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, green);
        glTranslatef(-0.47,0.5,0);
        gluSphere(quad,0.3,100,100);
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, blue);
        glTranslatef(-0.5,-0.5,0);
        gluSphere(quad,0.3,100,100);

        glewFun.BindBuffer(GL_PIXEL_PACK_BUFFER, pbos);
        glReadPixels(0, 0, image_width, image_height, GL_BGRA, GL_UNSIGNED_BYTE, NULL);

        this->RunKernel();

        glDisable(GL_DEPTH_TEST);
        glDisable(GL_LIGHTING);
        glEnable(GL_TEXTURE_2D);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

        glewFun.BindBuffer(GL_PIXEL_UNPACK_BUFFER, pbod);
        glBindTexture(GL_TEXTURE_2D, texscreen);
        glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0,image_width, image_height,GL_BGRA, GL_UNSIGNED_BYTE, NULL);

        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();
        glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

        glMatrixMode( GL_MODELVIEW);
        glLoadIdentity();

        glViewport(0, 0, image_width, image_height);

        glBegin(GL_QUADS);

           glTexCoord2f(0.0, 0.0);
           glVertex3f(-1.0, -1.0, 0.5);

           glTexCoord2f(1.0, 0.0);
           glVertex3f(1.0, -1.0, 0.5);

           glTexCoord2f(1.0, 1.0);
           glVertex3f(1.0, 1.0, 0.5);

           glTexCoord2f(0.0, 1.0);
           glVertex3f(-1.0, 1.0, 0.5);

        glEnd();

        glMatrixMode(GL_PROJECTION);
        glPopMatrix();

        glDisable(GL_TEXTURE_2D);
        glewFun.BindBuffer(GL_PIXEL_PACK_BUFFER, 0);
        glewFun.BindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);

    }
}

void OGLPP::wheelEvent(QWheelEvent * qwe)
{
    if (qwe->delta()>0) fzTr+=0.1;
    if (qwe->delta()<0) fzTr-=0.1;
    this->repaint();;
}

void OGLPP::mousePressEvent(QMouseEvent *qme)
{
    qpoint=qme->pos();
    this->repaint();;
}

void OGLPP::mouseMoveEvent(QMouseEvent *qme)
{
    fxRot+=180*(GLfloat)(qme->y()-qpoint.y())/this->height();
    fyRot+=180*(GLfloat)(qme->x()-qpoint.x())/this->width();
    this->repaint();;

    qpoint=qme->pos();
}

void OGLPP::animate()
{
    fyRot+=2;
    this->repaint();
}

void OGLPP::framecount()
{
    if (qlsd) qlsd->display(iCountSec);
    iCountSec=0;
}

void OGLPP::GenInit()
{
    Cleanup(-500);
    if ((this->uiCurPlat<0) || (this->uiCurDev<0)) return;

    unsigned int size = image_width * image_height * 4 * sizeof(GLubyte);

    glewFun.GenBuffers(1,&pbos);
    glewFun.BindBuffer(GL_ARRAY_BUFFER,pbos);
    glewFun.BufferData(GL_ARRAY_BUFFER,size,NULL, GL_DYNAMIC_DRAW);
    glewFun.BindBuffer(GL_ARRAY_BUFFER,0);

    glewFun.GenBuffers(1,&pbod);
    glewFun.BindBuffer(GL_ARRAY_BUFFER,pbod);
    glewFun.BufferData(GL_ARRAY_BUFFER,size,NULL, GL_DYNAMIC_DRAW);
    glewFun.BindBuffer(GL_ARRAY_BUFFER,0);

    glGenTextures(1, &texscreen);
    glBindTexture(GL_TEXTURE_2D, texscreen);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image_width, image_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

    #if defined (__APPLE__) || defined (MACOSX)
        CGLContextObj kCGLContext = CGLGetCurrentContext();
        CGLShareGroupObj kCGLShareGroup = CGLGetShareGroup(kCGLContext);
        cl_context_properties props[] =
        {
           CL_CONTEXT_PROPERTY_USE_CGL_SHAREGROUP_APPLE, (cl_context_properties)kCGLShareGroup,
           0
        };
        clc = clCreateContext(props, 0,0, NULL, NULL, &iErr);
    #else
        #ifdef UNIX
            cl_context_properties props[] =
            {
                CL_GL_CONTEXT_KHR, (cl_context_properties)glXGetCurrentContext(),
                CL_GLX_DISPLAY_KHR, (cl_context_properties)glXGetCurrentDisplay(),
                CL_CONTEXT_PLATFORM, (cl_context_properties)noclInf.clpi[0],
                0
            };
            clc = clCreateContext(props, 1, &noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev], NULL, NULL, &iErr);
            #else // Win32
                cl_context_properties props[] =
                {
                    CL_GL_CONTEXT_KHR, (cl_context_properties)wglGetCurrentContext(),
                    CL_WGL_HDC_KHR, (cl_context_properties)wglGetCurrentDC(),
                    CL_CONTEXT_PLATFORM, (cl_context_properties)noclInf.clpi[this->uiCurPlat],
                    0
                };
                clc = clCreateContext(props, 1, &noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev], NULL, NULL, &iErr);
        #endif
    #endif
    ERRORVOID(iErr);

    clcq=clCreateCommandQueue(clc,noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev],0,&iErr);
    ERRORVOID(iErr);

    cl_pbo[0]=clCreateFromGLBuffer(clc,CL_MEM_READ_ONLY,pbos,&iErr);
    ERRORVOID(iErr);
    cl_pbo[1]=clCreateFromGLBuffer(clc,CL_MEM_WRITE_ONLY,pbod,&iErr);
    ERRORVOID(iErr);

    QFile qfile(":/cl/PostprocessGL.cl");
    if (!qfile.open(QIODevice::ReadOnly))
    {
        return;
    }

    char *cSource;
    QByteArray qba=qfile.readAll();
    cSource=qba.data();
    size_t lenSource=qba.length();
    qfile.close();

    clp=clCreateProgramWithSource(clc,1,(const char **) &cSource,&lenSource,&iErr);
    ERRORVOID(iErr);

    iErr=clBuildProgram(clp, 0, NULL, "-cl-fast-relaxed-math", NULL, NULL);
    ERRORVOID(iErr);

    clk=clCreateKernel(clp,"postprocess",&iErr);
    ERRORVOID(iErr);

    LoadFun(&glewFun,this);

    iErr=clSetKernelArg(clk,0,sizeof(cl_mem),(void *)&(cl_pbo[0]));
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,1,sizeof(cl_mem),(void *)&(cl_pbo[1]));
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,2,sizeof(image_width),&image_width);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,3,sizeof(image_height),&image_height);
    ERRORVOID(iErr);

    this->brun=true;
    this->draw(1.0f);
}

void OGLPP::RunKernel()
{
    iCountSec++;

    glFlush();
    iErr=clEnqueueAcquireGLObjects(clcq,2, cl_pbo, 0, NULL, NULL);
    ERRORVOID(iErr);

    size_t szLocalWorkSize[] = {16,16};
    size_t szGlobalWorkSize[] = {0,0};
    szGlobalWorkSize[0]=(size_t)((size_t)(image_width/16)*16+16);
    szGlobalWorkSize[1]=(size_t)((size_t)(image_height/16)*16+16);
    cl_int radius=4;
    cl_int tilew =  (cl_int)szLocalWorkSize[0]+(2*radius);
    cl_float threshold = 0.8f;
    cl_float highlight = 4.0f;

    iErr=clSetKernelArg(clk,4,sizeof(tilew),&tilew);
    ERRORVOID(iErr);

    iErr=clSetKernelArg(clk,5,sizeof(radius),&radius);
    ERRORVOID(iErr);

    iErr=clSetKernelArg(clk,6,sizeof(threshold),&threshold);
    ERRORVOID(iErr);

    iErr=clSetKernelArg(clk,7,sizeof(highlight),&highlight);
    ERRORVOID(iErr);

    iErr=clSetKernelArg(clk, 8, (szLocalWorkSize[0]+(2*16))*(szLocalWorkSize[1]+(2*16))*sizeof(int), NULL);
    ERRORVOID(iErr);

    iErr= clEnqueueNDRangeKernel(clcq, clk, 2, NULL,szGlobalWorkSize, szLocalWorkSize,0, NULL, NULL);
    ERRORVOID(iErr);

    iErr=clEnqueueReleaseGLObjects(clcq,2, cl_pbo, 0, NULL, NULL);
    ERRORVOID(iErr);
    iErr=clFinish(clcq);
    ERRORVOID(iErr);


}

cl_int OGLPP::Cleanup(cl_int iErr)
{

    if (iErr<0)
    {
        if (iErr!=-500)
        {
            _noclGetNameErr(iErr,cErr);
            QMessageBox::critical(this,"Error!",QString(cErr));
        }
        if (pbos)
        {
            glewFun.BindBuffer(GL_ARRAY_BUFFER, pbos);
            glewFun.DeleteBuffers(1, &pbos);
            pbos=0;
        }
        if (pbod)
        {
            glewFun.BindBuffer(GL_ARRAY_BUFFER, pbod);
            glewFun.DeleteBuffers(1, &pbod);
            pbod=0;
        }
        if (texscreen)
        {
            glDeleteTextures(1,&texscreen);
            texscreen = 0;
        }
        if(clk)clReleaseKernel(clk);
        if(clp)clReleaseProgram(clp);
        if(cl_pbo[0])clReleaseMemObject(cl_pbo[0]);
        if(cl_pbo[1])clReleaseMemObject(cl_pbo[1]);
        if(clcq)clReleaseCommandQueue(clcq);
        if(clc)clReleaseContext(clc);
    }
    return iErr;
}
