#include "oglwdgt.h"

#define ERRORVOID(err) Cleanup(err); if(err!=CL_SUCCESS) return;

OGLWdgt::OGLWdgt(QWidget *parent) :
    QGLWidget(parent)
{
    fxRot=0.0f;
    fyRot=0.0f;
    fzRot=0.0f;
    fx=0.0f;
    fanim=0.0f;
    fanimspeed=0.03f;
    ffreq=4.0f;
    vbo=0;
    fzTr=-2.5f;
    brun=false;
    mesh_width = 256;
    mesh_height = 256;
    iErr=0;
    vbo_cl=NULL;
    clk=NULL;
    clp=NULL;
    clcq=NULL;
    vbo_cl=NULL;
    clc=NULL;

    qlsd=NULL;

    uiCurDev=-1;
    uiCurPlat=-1;

    iCountSec=0;

    memset(&noclInf,0,sizeof(noclInf));
    if (_noclGetInfo(&noclInf)==CL_SUCCESS)
    {
    }
}


void OGLWdgt::initializeGL()
{
    qglClearColor(Qt::black);
    LoadFun(&glewFun,this);
    glEnable(GL_DEPTH_TEST);
    glClearColor(0.0,0.0,0.0,1);
}

void OGLWdgt::resizeGL(int nWidth,int nHeight)
{
    glViewport(0,0,(GLsizei)nWidth,(GLsizei)nHeight);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, this->width()/this->height(),0.1, 10.0);
}

void OGLWdgt::draw(float fxx)
{
    fx=fxx;
    this->updateGL();
}

void OGLWdgt::paintGL()
{
    if (fx==0)
    {
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    }
    if (fx==1)
    {
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glTranslatef(0,0,fzTr);
        glRotatef(fxRot,1.0,0.0,0.0);
        glRotatef(fyRot,0.0,1.0,0.0);
        glRotatef(fzRot,0.0,0.0,1.0);
        glewFun.BindBuffer(GL_ARRAY_BUFFER, vbo);
        glVertexPointer(4, GL_FLOAT, 0, 0);
        glEnableClientState(GL_VERTEX_ARRAY);
        glColor3f(0.0, 1.0, 0.5);
        glDrawArrays(GL_POINTS, 0, mesh_height*mesh_width);
        glDisableClientState(GL_VERTEX_ARRAY);

    }
}

void OGLWdgt::wheelEvent(QWheelEvent * qwe)
{
    if (qwe->delta()>0) fzTr+=0.1;
    if (qwe->delta()<0) fzTr-=0.1;
    this->repaint();;
}

void OGLWdgt::mousePressEvent(QMouseEvent *qme)
{
    qpoint=qme->pos();
    this->repaint();;
}

void OGLWdgt::mouseMoveEvent(QMouseEvent *qme)
{
    fxRot+=180*(GLfloat)(qme->y()-qpoint.y())/this->height();
    fyRot+=180*(GLfloat)(qme->x()-qpoint.x())/this->width();
    this->repaint();;

    qpoint=qme->pos();
}

void OGLWdgt::animate()
{
    if (brun==true)
    {
        fanim+=fanimspeed;
        this->RunKernel();
    }
    this->repaint();
}

void OGLWdgt::framecount()
{
    if (qlsd) qlsd->display(iCountSec);
    iCountSec=0;
}

void OGLWdgt::GenInit()
{
    if ((this->uiCurPlat<0) || (this->uiCurDev<0)) return;
    this->Cleanup(-500);
    cl_context_properties props[]=
    {
        CL_CONTEXT_PLATFORM,(cl_context_properties)noclInf.clpi[this->uiCurPlat],
        0
    };
    clc=clCreateContext(props,1,&noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev],NULL,NULL,&iErr);
    ERRORVOID(iErr);

    clcq=clCreateCommandQueue(clc,noclInf.noclPlt[this->uiCurPlat].cldi[this->uiCurDev],0,&iErr);
    ERRORVOID(iErr);

    QFile qfile(":/cl/simplGL.cl");
    if (!qfile.open(QIODevice::ReadOnly))
    {
        return;
    }

    char *cSource;
    QByteArray qba=qfile.readAll();
    cSource=qba.data();
    size_t lenSource=qba.length();
    qfile.close();

    clp=clCreateProgramWithSource(clc,1,(const char **) &cSource,&lenSource,&iErr);
    ERRORVOID(iErr);

    iErr=clBuildProgram(clp, 0, NULL, "-cl-fast-relaxed-math", NULL, NULL);
    ERRORVOID(iErr);

    clk=clCreateKernel(clp,"sine_wave",&iErr);
    ERRORVOID(iErr);

    LoadFun(&glewFun,this);

    glewFun.GenBuffers(1,&vbo);
    glewFun.BindBuffer(GL_ARRAY_BUFFER, vbo);

    unsigned int size = mesh_width * mesh_height * 4 * sizeof(float);

    glewFun.BufferData(GL_ARRAY_BUFFER, size, NULL, GL_STREAM_DRAW);

    vbo_cl=clCreateBuffer(clc,CL_MEM_WRITE_ONLY,size,NULL,&iErr);

    iErr=clSetKernelArg(clk,0,sizeof(cl_mem),(void *)&vbo_cl);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,1,sizeof(unsigned int),(void *)&mesh_width);
    ERRORVOID(iErr);
    iErr=clSetKernelArg(clk,2,sizeof(unsigned int),(void *)&mesh_height);
    ERRORVOID(iErr);

    this->RunKernel();
    this->draw(1.0f);
    this->brun=true;
}

void OGLWdgt::RunKernel()
{
    iCountSec++;

    iErr=clSetKernelArg(clk,3,sizeof(float),(void *)&fanim);
    ERRORVOID(iErr);

    iErr=clSetKernelArg(clk,4,sizeof(float),(void *)&ffreq);
    ERRORVOID(iErr);

    size_t szGlobalWorkSize[] = {mesh_width, mesh_height};
    iErr=clEnqueueNDRangeKernel(clcq,clk,2,NULL,szGlobalWorkSize, NULL, 0,0,0);
    ERRORVOID(iErr);
    //glewFun.BindBuffer(GL_ARRAY_BUFFER, vbo);
    void* ptr = glewFun.MapBuffer(GL_ARRAY_BUFFER, GL_WRITE_ONLY);

    iErr=clEnqueueReadBuffer(clcq,vbo_cl,CL_TRUE,0, sizeof(float) * 4 * mesh_height * mesh_width, ptr, 0, NULL, NULL);
    ERRORVOID(iErr);

    glewFun.UnmapBuffer(GL_ARRAY_BUFFER);
}

cl_int OGLWdgt::Cleanup(cl_int iErr)
{

    if (iErr<0)
    {
        if (iErr!=-500)
        {
            _noclGetNameErr(iErr,cErr);
            QMessageBox::critical(this,"Error!",QString(cErr));
        }
        if(clk)clReleaseKernel(clk);
        if(clp)clReleaseProgram(clp);
        if(clcq)clReleaseCommandQueue(clcq);
        if(vbo)
        {
            glewFun.BindBuffer(1, vbo);
            glewFun.DeleteBuffers(1, &vbo);
            vbo = 0;
        }
        if(vbo_cl)clReleaseMemObject(vbo_cl);
        if(clc)clReleaseContext(clc);
    }
    return iErr;
}
